use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use prost::Message;

///
pub struct Service {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
}

impl std::fmt::Debug for Service {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("beteran-backend-server-edge::betting::service::Service")
      .finish()
  }
}

impl Service {
  ///
  pub fn new(connection_broker: nats::asynk::Connection, queue_broker: String) -> Service {
    Service {
      connection_broker,
      queue_broker,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(self.list_betting_history(),).map(|_| ())
  }

  fn get_client_in_header(
    &self,
    message: &nats::asynk::Message,
  ) -> Result<bpr::models::core::network::Client, bcr::error::rpc::Error> {
    match &message.headers {
      Some(headers) => {
        let client = match headers.get(bpr::c2se::core::network::HEADER_CLIENT) {
          Some(c) => {
            let msg = base64::decode(c).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?;
            bpr::models::core::network::Client::decode(msg.as_slice()).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?
          }
          None => {
            return Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
              message: "invalid client information".to_string(),
            }));
          }
        };

        Ok(client)
      }
      None => Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
        message: "invalid header".to_string(),
      })),
    }
  }

  async fn list_betting_history(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::api::betting::SUBJECT_LIST_BETTING_HISTORY,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req =
          bpr::c2se::api::betting::ListBettingHistoryRequest::decode(message.data.as_slice())
            .map_err(|e| {
              bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
                message: format!("invalid request: {}", e),
              })
            })?;

        let ss_list_betting_history_req = bpr::ss::api::betting::ListBettingHistoryRequest {
          client: Some(client.clone()),
          request: Some(
            bpr::ss::api::betting::list_betting_history_request::Request {
              pagination: req.pagination,
              sorts: req.sorts,
              search: match req.search {
                Some(s) => Some(
                  bpr::ss::api::betting::list_betting_history_request::request::Search {
                    member_id: s.member_id,
                    vendor_id: s.vendor_id,
                    vendor_ids: s.vendor_ids,
                    vendor_name: s.vendor_name,
                    vendor_name_like: s.vendor_name_like,
                    game_id: s.game_id,
                    game_ids: s.game_ids,
                    game_name: s.game_name,
                    game_name_like: s.game_name_like,
                    game_category: s.game_category,
                    game_category_like: s.game_category_like,
                    game_type: s.game_type,
                    game_type_like: s.game_type_like,
                    currency: s.currency,
                    currency_like: s.currency_like,
                    key: s.key,
                    key_like: s.key_like,
                    ref_id: s.ref_id,
                    ref_id_like: s.ref_id_like,
                    o_ref_id: s.o_ref_id,
                    o_ref_id_like: s.o_ref_id_like,
                    group_key: s.group_key,
                    group_key_like: s.group_key_like,
                    is_bonus: s.is_bonus,
                    is_promo: s.is_promo,
                    is_jackpot: s.is_jackpot,
                    site_username: s.site_username,
                    site_username_like: s.site_username_like,
                    betting_type: s.betting_type,
                    betting_type_like: s.betting_type_like,
                    category: s.category,
                    category_like: s.category_like,
                  },
                ),
                None => None,
              },
            },
          ),
        };

        let ss_list_betting_history_res_msg = self
          .connection_broker
          .request(
            bpr::ss::api::betting::SUBJECT_LIST_BETTING_HISTORY,
            ss_list_betting_history_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_list_betting_history_res =
          bpr::ss::api::betting::ListBettingHistoryResponse::decode(
            ss_list_betting_history_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_list_betting_history_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_list_betting_history_res.result {
          let mut usernames: Vec<String> = vec![];
          for b in &r.betting_history {
            let username = b.site_username.clone();
            if !usernames.contains(&username) {
              usernames.push(username);
            }
          }

          let members = if !usernames.is_empty() {
            let ss_list_members_req = bpr::ss::member::ListMembersRequest {
              client: Some(client.clone()),
              request: Some(bpr::ss::member::list_members_request::Request {
                pagination: None,
                sorts: vec![],
                search: Some(bpr::ss::member::list_members_request::request::Search {
                  site_id: None,
                  member_class_id: None,
                  member_level_id: None,
                  parent_member_id: None,
                  username: None,
                  usernames,
                  username_like: None,
                  nickname: None,
                  nicknames: vec![],
                  nickname_like: None,
                  mobile_phone_number_like: None,
                  last_signined_ip: None,
                  state: None,
                  deleted_at: None,
                }),
              }),
            };

            let ss_list_members_res_msg = self
              .connection_broker
              .request(
                bpr::ss::member::SUBJECT_LIST_MEMBERS,
                ss_list_members_req.encode_to_vec(),
              )
              .await
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })?;

            let ss_list_members_res =
              bpr::ss::member::ListMembersResponse::decode(ss_list_members_res_msg.data.as_slice())
                .map_err(|e| {
                  bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                    code: bpr::protobuf::rpc::Error::SERVER_00,
                    message: format!("server {}", e),
                    data: None,
                  })
                })?;

            if let Some(e) = ss_list_members_res.error {
              return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              }));
            }

            match ss_list_members_res.result {
              Some(rr) => rr.members,
              None => vec![],
            }
          } else {
            vec![]
          };

          let mut betting_history: Vec<bpr::models::api::betting::BettingHistoryModel> = vec![];
          for b in r.betting_history {
            let member = members
              .iter()
              .find(|v| v.username.eq(&b.site_username))
              .cloned();
            betting_history.push(bpr::models::api::betting::BettingHistoryModel {
              id: b.id,
              vendor_id: b.vendor_id,
              vendor_name: b.vendor_name,
              game_id: b.game_id,
              game_name: b.game_name,
              game_category: b.game_category,
              game_type: b.game_type,
              currency: b.currency,
              cash: b.cash,
              before_cash: b.before_cash,
              after_cash: b.after_cash,
              group_key: b.group_key,
              detail: b.detail,
              is_bonus: b.is_bonus,
              is_promo: b.is_promo,
              is_jackpot: b.is_jackpot,
              site_username: b.site_username,
              key: b.key,
              ref_id: b.ref_id,
              o_ref_id: b.o_ref_id,
              betting_type: b.betting_type,
              category: b.category,
              created_at: b.created_at,
              utc_created_at: b.utc_created_at,
              member,
            });
          }

          message
            .respond(
              bpr::c2se::api::betting::ListBettingHistoryResponse {
                error: None,
                result: Some(
                  bpr::c2se::api::betting::list_betting_history_response::Result {
                    total_count: r.total_count,
                    betting_history,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::api::betting::ListBettingHistoryResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }
}
